@@ -2,7 +2,7 @@
 # Dancer params() keyword, and returns then when its param() method is called,
 # so that we can feed it to CGI::FormBuilder:
 package Dancer::Plugin::SimpleCRUD::ParamsObject;
-sub new { 
+sub new {
     my ($class, $params) = @_; 
     return bless { params => $params }, $class;
 }
@@ -31,11 +31,8 @@ use Dancer qw(:syntax);
 use Dancer::Plugin::Database;
 use HTML::Table::FromDatabase;
 use CGI::FormBuilder;
-use SQL::Abstract;
-
-our $VERSION = '0.02';
-
 
+our $VERSION = '0.03';
 
 =head1 NAME
 
@@ -46,7 +43,7 @@ Dancer::Plugin::SimpleCRUD - very simple CRUD (create/read/update/delete)
 
 A plugin for Dancer web applications, to use a  few lines of code to create
 appropriate routes to support creating/editing/deleting/viewing records within a
-database table.  Uses L<HTML::FormFu> to generate, process and validate forms,
+database table.  Uses L<CGI::FormBuilder> to generate, process and validate forms,
 L<Dancer::Plugin::Database> for database interaction and
 L<HTML::Table::FromDatabase> to display lists of records.
 
@@ -72,7 +69,7 @@ L<HTML::Table::FromDatabase> to display lists of records.
         record_title => 'Widget',
         prefix => '/widgets',
         db_table => 'widgets',
-        field_labels => {
+        labels => {
             country => 'Country of Origin',
             type    => 'Widget Type', 
         },  
@@ -137,7 +134,7 @@ database connection details in your config file will be used - this is often
 what you want, so unless your app is dealing with multiple DBs, you probably
 won't need to worry about this option.
 
-=item <field_labels> (optional)
+=item <labels> (optional)
 
 A hashref of field_name => 'Label', if you want to provide more user-friendly
 labels for some or all fields.  As we're using CGI::FormBuilder, it will do a
@@ -146,7 +143,7 @@ field named C<first_name> will be shown as C<First Name>.
 
 =item C<validation> (optional)
 
-A hashref of validation criteria which should be passed to HTML::FormFu.
+A hashref of validation criteria which should be passed to L<CGI::FormBuilder>.
 
 =item C<acceptable_values> (optional)
 
@@ -165,6 +162,12 @@ columns in the database table, with the exception of the key column.
 
 Specify an arrayref of fields which should not be editable.
 
+=item C<required> (optional)
+
+Specify an arrayref of fields which must be completed.  If this is not provided,
+DWIMmery based on whether the field is set to allow null values in the database
+will be used - i.e. if that column can contain null, then it doesn't have to be
+completed, otherwise, it does.
 
 =item C<deletable>
 
@@ -172,7 +175,6 @@ Specify whether to support deleting records.  If set to a true value, a route
 will be created for C</prefix/delete/:id> to delete the record with the ID
 given, and the edit form will have a "Delete $record_title" button.
 
-
 =cut
 
 sub simple_crud {
@@ -198,7 +200,7 @@ sub simple_crud {
     if ($db_type ne 'MySQL') {
         warn "This module has so far only been tested with MySQL databases.";
     }
-    
+
     # Accepta deleteable as a synonym for deletable
     $args{deletable} = delete $args{deleteable}
         if !exists $args{deletable} && exists $args{deleteable};
@@ -218,219 +220,18 @@ sub simple_crud {
     }
 
     # OK, create a route handler to deal with adding/editing:
-    my $handler = sub {
-        my $params = params;
-        my $id = $params->{id};
+    my $handler = sub { _create_add_edit_route(\%args, $table_name, $key_column); };
 
-        my $dbh = database($args{db_connection_name});
-        
-        my $default_field_values;
-        if ($id) {
-            my $record = database->selectrow_hashref(
-                "select * from $table_name where $key_column = ?",
-                {}, $id
-            );
-            $default_field_values = $record;
-        }
-
-        # Find out about table columns:
-        my $all_table_columns = _find_columns($dbh, $args{db_table}); 
-        my @editable_columns;
-        # Now, find out which ones we can edit.
-        if ($args{editable_columns}) {
-            # We were given an explicit list of fields we can edit, so this is
-            # easy:
-            @editable_columns = @{ $args{editable_columns} };
-        } else {
-            # OK, take all the columns from the table, except the key field:
-            @editable_columns = grep { $_ ne $key_column } 
-                map { $_->{COLUMN_NAME} } @$all_table_columns;
-        }
-
-        # Some DWIMery: if we don't have a validation rule specified for a
-        # field, and it's pretty clear what it is supposed to be, just do it:
-        my $validation = $args{validation} || {};
-        for my $field (grep { $_ ne $key_column } @editable_columns) 
-        {
-            next if $validation->{$field};
-            if ($field =~ /email/) {
-                $validation->{$field} = 'EMAIL';
-            }
-        }
-
-        # More DWIMmery: if the user hasn't supplied a list of required fields,
-        # work out what fields are required by whether they're nullable in the
-        # DB:
-        my %required_fields;
-        if (exists $args{required}) {
-            $required_fields{$_}++ for @{ $args{required} };
-        } else {
-            $_->{NULLABLE} || $required_fields{ $_->{COLUMN_NAME} }++
-                for @$all_table_columns;
-        }
-
-        # If the user didn't supply a list of acceptable values for a field, but
-        # it's an ENUM column, use the possible values declared in the ENUM.
-        # Also remember field types for easy reference later
-        my %constrain_values;
-        my %field_type;
-        for my $field (@$all_table_columns) {
-            my $name = $field->{COLUMN_NAME};
-            $field_type{$name} = $field->{TYPE_NAME};
-            if (my $values_specified = $args{acceptable_values}->{$name}) {
-                $constrain_values{$name} = $values_specified;
-            } elsif (my $values_from_db = $field->{mysql_values}) {
-                $constrain_values{$name} = $values_from_db;
-            }
-        }
-        
-        # Only give CGI::FormBuilder our fake CGI object if the form has been
-        # POSTed to us already; otherwise, it will ignore default values from
-        # the DB, it seems.
-        my $paramsobj = request->{method} eq 'POST' ?
-            Dancer::Plugin::SimpleCRUD::ParamsObject->new({params()}) : undef;
-
-        my $form = CGI::FormBuilder->new(
-            fields => \@editable_columns,
-            params => $paramsobj,
-            values => $default_field_values,
-            validate => $validation,
-            method => 'post',
-            action => $args{prefix} . 
-                (params->{id} ? '/edit/' . params->{id} : '/add'),
-        );
-        for my $field (@editable_columns) {
-            my %field_params = (
-                name => $field,
-                value => $default_field_values->{$field} || '',
-            );
-            if (my $label = $args{labels}->{$field}) {
-                $field_params{label} = $label;
-            }
-            if (my $validation = $args{validation}->{$field}) {
-                $field_params{validate} = $validation;
-            }
-
-            $field_params{required} = $required_fields{$field};
-
-            if ($constrain_values{$field}) {
-                $field_params{options} = $constrain_values{$field};
-            }
-
-            # Normally, CGI::FormBuilder can guess the type of field perfectly,
-            # but give it some extra DWIMmy help:
-            if ($field =~ /pass(?:wd|word)?$/i) {
-                $field_params{type} = 'password';
-            }
-
-            # use a <textarea> for large text fields.
-            if ($field_type{$field} eq 'TEXT') {
-                $field_params{type} = 'textarea';
-            }
-
-            # OK, add the field to the form:
-            $form->field(%field_params);
-        }
-
-
-        # Now, if all is OK, go ahead and process:
-        if (request->{method} eq 'POST' &&  $form->submitted && $form->validate) 
-        {
-            # Assemble a hash of only fields from the DB (if other fields were
-            # submitted with the form which don't belong in the DB, ignore them)
-            my %params;
-            $params{$_} = params->{$_} for @editable_columns;
-            my $sql = SQL::Abstract->new( quote_char => '`' );
-            my ($statement, @bind_values);
-            my $verb;
-            if (exists params->{$key_column}) {
-                # We're editing an existing record
-                ($statement, @bind_values) = $sql->update($table_name, \%params, 
-                    { $key_column => params->{$key_column} }
-                );
-                $verb = 'update';
-            } else {
-                ($statement, @bind_values) = $sql->insert($table_name, \%params);
-                $verb = 'create new';
-            }
-
-            if (database->do($statement, undef, @bind_values)) {
-                # Redirect to the list page
-                # TODO: pass a param to cause it to show a message?
-                redirect $args{prefix};
-                return;
-            } else {
-                # TODO: better error handling
-                return "<p>Unable to $verb $args{record_title}</p>";
-            }
-
-        } else {
-            return engine('template')->apply_layout($form->render);
-        }
-    };
     Dancer::Logger::debug("Setting up routes for $args{prefix}/add etc");
     any ['get','post'] => "$args{prefix}/add"      => $handler;
     any ['get','post'] => "$args{prefix}/edit/:id" => $handler;
 
     # And a route to list records already in the table:
-    get "$args{prefix}" => sub {
-        # TODO: handle pagination
-        my $dbh = database($args{db_connection_name});
-        my $sth = $dbh->prepare("select *, $key_column as actions from $table_name");
-        $sth->execute
-            or die "Failed to query for records in $table_name - "
-                . database->errstr;
-        my $table = HTML::Table::FromDatabase->new(
-            -sth => $sth,
-            -border => 1,
-            -callbacks => [
-                {
-                    column => 'actions',
-                    transform => sub {
-                        my $id = shift;
-                        my $action_links;
-                        my $edit_url = "$args{prefix}/edit/$id";
-                        $action_links .= 
-                            qq[<a href="$edit_url" class="edit_link">Edit</a>];
-                        if ($args{deletable}) {
-                            my $del_url = "$args{prefix}/delete/$id";
-                            $action_links .=
-                                qq[ / <a href="$del_url" class="delete_link"]
-                               .qq[ onclick="delrec('$id'); return false;">]
-                               .qq[Delete</a>];
-                        }
-                        return $action_links;
-                    },
-                },
-            ],
-        );
-        my $html =  $table->getTable;
-        $html .= sprintf '<a href="%s">Add a new %s</a></p>',
-            $args{prefix} . '/add', $args{record_title};
-
-        # Append a little Javascript which asks for confirmation that they'd
-        # like to delete the record, then makes a POST request via a hidden
-        # form.  This could be made AJAXy in future.
-        $html .= <<DELETEJS;
-<form name="deleteform" method="post" action="$args{prefix}/delete">
-<input name="record_id" type="hidden">
-</form>
-<script language="Javascript">
-function delrec(record_id) {
-    if (confirm('Confirm you wish to delete this record?')) {
-        document.deleteform.rowid.value = record_id;
-        document.deleteform.submit();
-    }
-}
-</script>
-
-DELETEJS
-        return engine('template')->apply_layout($html);
-    };
+    my $list_handler = sub { _create_list_handler(\%args, $table_name, $key_column); };
+    get  "$args{prefix}"        => $list_handler;
 
     # If we should allow deletion of records, set up routes to handle that,
     # too.
-
     if ($args{deletable}) {
         # A route for GET requests, to present a "Do you want to delete this"
         # message with a form to submit (this is only for browsers which didn't
@@ -447,14 +248,14 @@ Do you really wish to delete this record?
 <input type="submit" value="Delete record">
 </form>
 CONFIRMDELETE
-        
+
         };
 
         # A route for POST requests, to actually delete the record
         post qr[$args{prefix}/delete/?(.+)?$] => sub {
             my ($id) = params->{record_id} || splat;
-            my $sql = "DELETE FROM $table_name where $key_column = ?";
-            database($args{db_connection_name})->do($sql, undef, $id);
+            database->quick_delete($table_name, { $key_column => $id })
+                or return "<p>Failed to delete!</p>";
             redirect $args{prefix};
         };
     }
@@ -465,6 +266,258 @@ register simple_crud => \&simple_crud;
 register_plugin;
 
 
+sub _create_add_edit_route {
+    my ($args, $table_name, $key_column) = @_;
+    my $params = params;
+    my $id = $params->{id};
+
+    my $dbh = database($args->{db_connection_name});
+
+    my $default_field_values;
+    if ($id) {
+        $default_field_values = database->quick_select(
+            $table_name, { $key_column => $id}
+        );
+    }
+
+    # Find out about table columns:
+    my $all_table_columns = _find_columns($dbh, $args->{db_table}); 
+    my @editable_columns;
+    # Now, find out which ones we can edit.
+    if ($args->{editable_columns}) {
+        # We were given an explicit list of fields we can edit, so this is
+        # easy:
+        @editable_columns = @{ $args->{editable_columns} };
+    } else {
+        # OK, take all the columns from the table, except the key field:
+        @editable_columns = grep { $_ ne $key_column } 
+          map { $_->{COLUMN_NAME} } @$all_table_columns;
+    }
+
+    # Some DWIMery: if we don't have a validation rule specified for a
+    # field, and it's pretty clear what it is supposed to be, just do it:
+    my $validation = $args->{validation} || {};
+        for my $field (grep { $_ ne $key_column } @editable_columns) 
+          {
+              next if $validation->{$field};
+              if ($field =~ /email/) {
+                  $validation->{$field} = 'EMAIL';
+              }
+          }
+
+    # More DWIMmery: if the user hasn't supplied a list of required fields,
+    # work out what fields are required by whether they're nullable in the
+    # DB:
+    my %required_fields;
+    if (exists $args->{required}) {
+        $required_fields{$_}++ for @{ $args->{required} };
+    } else {
+        $_->{NULLABLE} || $required_fields{ $_->{COLUMN_NAME} }++
+          for @$all_table_columns;
+    }
+
+    # If the user didn't supply a list of acceptable values for a field, but
+    # it's an ENUM column, use the possible values declared in the ENUM.
+    # Also remember field types for easy reference later
+    my %constrain_values;
+    my %field_type;
+    for my $field (@$all_table_columns) {
+        my $name = $field->{COLUMN_NAME};
+        $field_type{$name} = $field->{TYPE_NAME};
+        if (my $values_specified = $args->{acceptable_values}->{$name}) {
+            $constrain_values{$name} = $values_specified;
+        } elsif (my $values_from_db = $field->{mysql_values}) {
+            $constrain_values{$name} = $values_from_db;
+        }
+    }
+
+    # Only give CGI::FormBuilder our fake CGI object if the form has been
+    # POSTed to us already; otherwise, it will ignore default values from
+    # the DB, it seems.
+    my $paramsobj = request->{method} eq 'POST' ?
+      Dancer::Plugin::SimpleCRUD::ParamsObject->new({params()}) : undef;
+
+    my $form = CGI::FormBuilder->new(
+                                     fields => \@editable_columns,
+                                     params => $paramsobj,
+                                     values => $default_field_values,
+                                     validate => $validation,
+                                     method => 'post',
+                                     action => $args->{prefix} . 
+                                     (params->{id} ? '/edit/' . params->{id} : '/add'),
+                                    );
+    for my $field (@editable_columns) {
+        my %field_params = (
+                            name => $field,
+                            value => $default_field_values->{$field} || '',
+                           );
+        if (my $label = $args->{labels}->{$field}) {
+            $field_params{label} = $label;
+        }
+        if (my $validation = $args->{validation}->{$field}) {
+            $field_params{validate} = $validation;
+        }
+
+        $field_params{required} = $required_fields{$field};
+
+        if ($constrain_values{$field}) {
+            $field_params{options} = $constrain_values{$field};
+        }
+
+        # Normally, CGI::FormBuilder can guess the type of field perfectly,
+        # but give it some extra DWIMmy help:
+        if ($field =~ /pass(?:wd|word)?$/i) {
+            $field_params{type} = 'password';
+        }
+
+        # use a <textarea> for large text fields.
+        if ($field_type{$field} eq 'TEXT') {
+            $field_params{type} = 'textarea';
+        }
+
+        # OK, add the field to the form:
+        $form->field(%field_params);
+    }
+
+    # Now, if all is OK, go ahead and process:
+    if (request->{method} eq 'POST' &&  $form->submitted && $form->validate) 
+      {
+          # Assemble a hash of only fields from the DB (if other fields were
+          # submitted with the form which don't belong in the DB, ignore them)
+          my %params;
+          $params{$_} = params->{$_} for @editable_columns;
+          my $verb;
+          my $success;
+          if (exists params->{$key_column}) {
+              # We're editing an existing record
+              $success = database->quick_update($table_name, \%params, 
+                { $key_column => params->{$key_column} }
+              );
+              $verb = 'update';
+          } else {
+              $success = database->quick_insert($table_name, \%params);
+              $verb = 'create new';
+          }
+
+          if ($success) {
+              # Redirect to the list page
+              # TODO: pass a param to cause it to show a message?
+              redirect $args->{prefix};
+              return;
+          } else {
+              # TODO: better error handling - options to provide error templates
+              # etc
+              return "<p>Unable to $verb $args->{record_title}</p>";
+          }
+
+      } else {
+          return engine('template')->apply_layout($form->render);
+      }
+}
+
+
+
+sub _create_list_handler {
+    my ($args, $table_name, $key_column) = @_;
+
+    my $dbh = database($args->{db_connection_name});
+    my $columns = _find_columns($dbh, $table_name);
+    my $options = join("\n", map {
+        "<option value='$_->{COLUMN_NAME}'>$_->{COLUMN_NAME}</option>"
+    } @$columns);
+    my $html = <<"SEARCHFORM";
+ <p><form name="searchform" method="get">
+     Field:  <select name="searchfield">$options</select> &nbsp;&nbsp;
+     Query: <input name="q" type="text" size="30"/> &nbsp;&nbsp;
+     <input name="searchsubmit" type="submit" value="Search"/>
+ </form></p>
+SEARCHFORM
+
+    # TODO: handle pagination
+    # TODO: Fix me with more data types. Make this global?
+    my %known_type = (
+                      INT     => q{%s = %s},
+                      VARCHAR => q{%s LIKE %s},
+                     );
+    my $query  = "SELECT *, $key_column AS actions FROM $table_name";
+
+    if (params->{'q'}) {
+        my ($column_data) = grep {
+            lc $_->{COLUMN_NAME} eq lc params->{searchfield}
+        } @{ $columns };
+        debug("Searching on $column_data->{COLUMN_NAME} which is a "
+            . "$column_data->{TYPE_NAME}");
+
+        if ($column_data and
+                my $add_clause = $known_type{uc $column_data->{TYPE_NAME}})
+        {
+            $query .=  ' WHERE ' . sprintf $add_clause,
+                $dbh->quote_identifier(params->{searchfield}),
+                $dbh->quote('%' . params->{'q'} . '%');
+
+            $html .= sprintf (
+                "<p>Showing results from searching for '%s' in '%s'",
+                params->{'q'}, params->{searchfield}
+            );
+            $html .= qq[&mdash;<a href="$args->{prefix}">Reset search</a></p>];
+        }
+    }
+    debug("Running query: $query");
+    my $sth = $dbh->prepare($query);
+    $sth->execute()
+      or die "Failed to query for records in $table_name - "
+        . database->errstr;
+    my $table = HTML::Table::FromDatabase->new
+      (
+       -sth => $sth,
+       -border => 1,
+       -callbacks => [
+                      {
+                       column => 'actions',
+                       transform => sub {
+                           my $id = shift;
+                           my $action_links;
+                           my $edit_url = "$args->{prefix}/edit/$id";
+                           $action_links .= 
+                             qq[<a href="$edit_url" class="edit_link">Edit</a>];
+                           if ($args->{deletable}) {
+                               my $del_url = "$args->{prefix}/delete/$id";
+                               $action_links .=
+                                 qq[ / <a href="$del_url" class="delete_link"]
+                                   .qq[ onclick="delrec('$id'); return false;">]
+                                     .qq[Delete</a>];
+                           }
+                           return $action_links;
+                       },
+                      },
+                     ],
+      );
+
+    $html .= $table->getTable;
+    $html .= sprintf '<a href="%s">Add a new %s</a></p>',
+      $args->{prefix} . '/add', $args->{record_title};
+
+    # Append a little Javascript which asks for confirmation that they'd
+    # like to delete the record, then makes a POST request via a hidden
+    # form.  This could be made AJAXy in future.
+    $html .= <<DELETEJS;
+<form name="deleteform" method="post" action="$args->{prefix}/delete">
+<input name="record_id" type="hidden">
+</form>
+<script language="Javascript">
+function delrec(record_id) {
+    if (confirm('Confirm you wish to delete this record?')) {
+        document.deleteform.rowid.value = record_id;
+        document.deleteform.submit();
+    }
+}
+</script>
+
+DELETEJS
+    return engine('template')->apply_layout($html);
+}
+
+
 # Given a table name, return an arrayref of hashrefs describing each column in
 # the table.
 # Expect to see the following keys:
@@ -521,6 +574,14 @@ C<simple_crud>, in which case what you say goes.)
 
 David Precious, C<< <davidp@preshweb.co.uk> >>
 
+=head1 ACKNOWLEDGEMENTS
+
+Alberto Simões (ambs)
+
+WK
+
+Johnathan Barber
+
 
 =head1 BUGS
 
@@ -577,6 +638,10 @@ L<http://search.cpan.org/dist/Dancer-Plugin-SimpleCRUD/>
 
 =head1 ACKNOWLEDGEMENTS
 
+Alberto Simões (ambs)
+
+Jonathan Barber
+
 
 =head1 LICENSE AND COPYRIGHT
 
